/*
 * Arduino weather station with ST7735 color TFT display (128x160 pixel) and
 *   BME280 barometric pressure, temperature & humidity sensor.
 * This is a free software with NO WARRANTY.
 * https://simple-circuit.com/
 */

#include <ArduinoJson.h>       // for JSON parse to NodeMCU
#include <Adafruit_GFX.h>      // include Adafruit graphics library
#include <Adafruit_ST7735.h>   // include Adafruit ST7735 TFT library
#include <Adafruit_BME280.h>   // include Adafruit BME280 sensor library

#define TFT_RST   9      // TFT RST pin is connected to arduino pin 9
#define TFT_CS    10      // TFT CS  pin is connected to arduino pin 10
#define TFT_DC    8     // TFT DC  pin is connected to arduino pin 8

// initialize ST7735 TFT library
Adafruit_ST7735 tft = Adafruit_ST7735(TFT_CS, TFT_DC, TFT_RST);

// define device I2C address 0x77
#define BME280_I2C_ADDRESS  0x77

Adafruit_BME280  bme280;  // initialize Adafruit BME280 library

void setup(void)
{
  Serial.begin(9600);
  tft.initR(INITR_BLACKTAB);     // initialize a ST7735S chip, black tab
  tft.fillScreen(ST7735_BLACK);  // fill screen with black color
  tft.drawFastHLine(0, 30,  tft.width(), ST7735_WHITE);   // draw horizontal white line at position (0, 30)

  tft.setTextColor(ST7735_WHITE, ST7735_BLACK);  // set text color to white and black background
  tft.setTextSize(1);                 // text size = 1
  tft.setCursor(4, 0);               // move cursor to position (4, 0) pixel
  tft.print("ARDUINO + ST7735 TFT");
  tft.setCursor(19, 15);              // move cursor to position (19, 15) pixel
  tft.print("WEATHER STATION");

  // initialize the BME280 sensor
  if(!bme280.begin(BME280_I2C_ADDRESS))
  {  // connection error or device address wrong!
    tft.setTextColor(ST7735_RED, ST7735_BLACK);   // set text color to red and black background
    tft.setTextSize(2);         // text size = 2
    tft.setCursor(5, 76);       // move cursor to position (5, 76) pixel
    tft.print("Connection");
    tft.setCursor(35, 100);     // move cursor to position (35, 100) pixel
    tft.print("Error");
    while(1);  // stay here
  }

  tft.drawFastHLine(0, 76,  tft.width(), ST7735_WHITE);  // draw horizontal white line at position (0, 76)
  tft.drawFastHLine(0, 122,  tft.width(), ST7735_WHITE);  // draw horizontal white line at position (0, 122)
  tft.setTextColor(ST7735_RED, ST7735_BLACK);     // set text color to red and black background
  tft.setCursor(25, 39);              // move cursor to position (25, 39) pixel
  tft.print("TEMPERATURE =");
  tft.setTextColor(ST7735_CYAN, ST7735_BLACK);  // set text color to cyan and black background
  tft.setCursor(34, 85);              // move cursor to position (34, 85) pixel
  tft.print("HUMIDITY =");
  tft.setTextColor(ST7735_GREEN, ST7735_BLACK);  // set text color to green and black background
  tft.setCursor(34, 131);              // move cursor to position (34, 131) pixel
  tft.print("PRESSURE =");
  tft.setTextSize(2);                 // text size = 2

}

// main loop
void loop()
{
  char _buffer[8];
  // read temperature, humidity and pressure from the BME280 sensor
  float temp = bme280.readTemperature();    // get temperature in °C
  float humi = bme280.readHumidity();       // get humidity in rH%
  float pres = bme280.readPressure();       // get pressure in Pa

  // print temperature (in °C)
  if(temp < 0)    // if temperature < 0
    sprintf( _buffer, "-%02u.%02u", (int)abs(temp), (int)(abs(temp) * 100) % 100 );
  else            // temperature >= 0
    sprintf( _buffer, " %02u.%02u", (int)temp, (int)(temp * 100) % 100 );
  tft.setTextColor(ST7735_YELLOW, ST7735_BLACK);  // set text color to yellow and black background
  tft.setCursor(11, 54);
  tft.print(_buffer);
  tft.drawCircle(89, 56, 2, ST7735_YELLOW);  // print degree symbol ( ° )
  tft.setCursor(95, 54);
  tft.print("C");

  // 2: print humidity
  sprintf( _buffer, "%02u.%02u %%", (int)humi, (int)(humi * 100) % 100 );
  tft.setTextColor(ST7735_MAGENTA, ST7735_BLACK);  // set text color to magenta and black background
  tft.setCursor(23, 100);
  tft.print(_buffer);

  // 3: print pressure (in hPa)
  sprintf( _buffer, "%04u.%02u", (int)(pres/100), (int)((uint32_t)pres % 100) );
  tft.setTextColor(0xFD00, ST7735_BLACK);  // set text color to orange and black background
  tft.setCursor(3, 146);
  tft.print(_buffer);
  tft.setCursor(91, 146);
  tft.print("hPa");


  // NodeMCU comunication
  DynamicJsonDocument doc(512);

  // create the corresponding json object
  char json[] = "{\"temperature\": 0, \"humidity\": 0, \"pressure\": 0}";
  DeserializationError error = deserializeJson(doc, json);
  if (error) {
    Serial.print(F("deserialize_Json() failed: "));
    Serial.println(error.c_str());
  }
  doc["temperature"] = temp;
  doc["humidity"] = humi;
  doc["pressure"] = pres / 100;
  serializeJson(doc, Serial);

  delay(1000);    // wait a second

}

// end of code.
